/*
 *
 *  Don't Be Square - a Nintendo 64 tech demo
 *       by Sebastian Mihai, 2015
 *       http://sebastianmihai.com
 *
 */ 

#include <string>
#include <cmath>

#include <libdragon.h>

#define BLACK graphics_make_color(0x00, 0x00, 0x00, 0xFF)
#define RED graphics_make_color(0xFF, 0x00, 0x00, 0xFF)
#define GREEN graphics_make_color(0x00, 0xFF, 0x00, 0xFF)
#define BLUE graphics_make_color(0x00, 0x00, 0xFF, 0xFF)
#define WHITE graphics_make_color(0xFF, 0xFF, 0xFF, 0xFF)

// Pad buttons
#define A_BUTTON(a)     ((a) & 0x8000)
#define B_BUTTON(a)     ((a) & 0x4000)
#define Z_BUTTON(a)     ((a) & 0x2000)
#define START_BUTTON(a) ((a) & 0x1000)

// D-Pad
#define DU_BUTTON(a)    ((a) & 0x0800)
#define DD_BUTTON(a)    ((a) & 0x0400)
#define DL_BUTTON(a)    ((a) & 0x0200)
#define DR_BUTTON(a)    ((a) & 0x0100)

// Triggers
#define TL_BUTTON(a)    ((a) & 0x0020)
#define TR_BUTTON(a)    ((a) & 0x0010)

// Yellow C buttons
#define CU_BUTTON(a)    ((a) & 0x0008)
#define CD_BUTTON(a)    ((a) & 0x0004)
#define CL_BUTTON(a)    ((a) & 0x0002)
#define CR_BUTTON(a)    ((a) & 0x0001)

struct controller_data gKeys;

display_context_t _dc;

extern "C" {
	/* vblank callback */
	void vblCallback(void){
	}

	void init_n64(void) {
		init_interrupts();
		display_init(RESOLUTION_320x240, DEPTH_16_BPP, 2, GAMMA_NONE, ANTIALIAS_RESAMPLE);
		register_VI_handler(vblCallback);
		controller_init();
	}

	unsigned short get_pad(int pad) {
		// Read current controller status
		controller_scan();
		gKeys = get_keys_pressed();
		return (unsigned short)(gKeys.c[pad].data >> 16);
	}

	void clear_screen(void) {
		while (!(_dc = display_lock()));
		graphics_fill_screen(_dc, graphics_make_color(0x00, 0x00, 0x00, 0xFF));
	}

	void put_chr(char chr, unsigned int color, int x, int y) {
		char temp[4];
		temp[0] = chr;
		temp[1] = 0;
		graphics_set_color(color, 0);
		graphics_draw_text(_dc, x*8, y*8, temp);
	}

	void put_str(char *str, unsigned int color, int x, int y) {
		graphics_set_color(color, 0);
		graphics_draw_text(_dc, x*8, y*8, str);
	}
}


typedef struct Rectangle {
	Rectangle* parent;
	int x, y;
	int width, height;
	int dx, dy;
	int colour;
} Rectangle;

void draw_rectangle( Rectangle* rectangle ) {
	graphics_draw_box( 
		_dc, 
		rectangle->x, 
		rectangle->y, 
		rectangle->width, 
		rectangle->height, 
		rectangle->colour 
		);
}

int main() {
	
	Rectangle rect;
	int seed = 13;
	
    init_n64();
	
	clear_screen();
	while( 1 ) {
		
		rect.width = seed % 40;
		rect.height = seed % 35;
		rect.x = seed % 280;
		rect.y = seed % 200;
		rect.colour = graphics_make_color( seed * 3, seed * 13, seed * 19, 0xFF );
		draw_rectangle( &rect );
		
		put_str( (char*)"Don't Be Square demo  by Sebastian Mihai", WHITE, 0, 29 );
		
		// swap buffers
		display_show(_dc);
		
		seed = ((seed * 7) % 3) + 25;
	}


    return 0;
} // end of main function

