
#include <string>
#include <cmath>

#include <libdragon.h>

#define STARTX 14*8+4
#define STARTY 6*8+4
#define LENGTH 12*8
#define OFFSET 4*8

#define RED graphics_make_color(0xFF, 0x00, 0x00, 0xFF)
#define GREEN graphics_make_color(0x00, 0xFF, 0x00, 0xFF)
#define WHITE graphics_make_color(0xFF, 0xFF, 0xFF, 0xFF)

// Pad buttons
#define A_BUTTON(a)     ((a) & 0x8000)
#define B_BUTTON(a)     ((a) & 0x4000)
#define Z_BUTTON(a)     ((a) & 0x2000)
#define START_BUTTON(a) ((a) & 0x1000)

// D-Pad
#define DU_BUTTON(a)    ((a) & 0x0800)
#define DD_BUTTON(a)    ((a) & 0x0400)
#define DL_BUTTON(a)    ((a) & 0x0200)
#define DR_BUTTON(a)    ((a) & 0x0100)

// Triggers
#define TL_BUTTON(a)    ((a) & 0x0020)
#define TR_BUTTON(a)    ((a) & 0x0010)

// Yellow C buttons
#define CU_BUTTON(a)    ((a) & 0x0008)
#define CD_BUTTON(a)    ((a) & 0x0004)
#define CL_BUTTON(a)    ((a) & 0x0002)
#define CR_BUTTON(a)    ((a) & 0x0001)

struct controller_data gKeys;

display_context_t _dc;

volatile unsigned int gTicks = 0;

extern "C" {
/* vblank callback */
void vblCallback(void)
{
    gTicks++;
}

void init_n64(void)
{
    init_interrupts();
    display_init(RESOLUTION_320x240, DEPTH_16_BPP, 2, GAMMA_NONE, ANTIALIAS_RESAMPLE);
    register_VI_handler(vblCallback);
    controller_init();
}

unsigned short get_pad(int pad)
{
    // Read current controller status
    controller_scan();
    gKeys = get_keys_pressed();
    return (unsigned short)(gKeys.c[pad].data >> 16);
}

void delay(int cnt)
{
    unsigned int then = gTicks + cnt;
    while (then > gTicks) ;
}

void clear_screen(void)
{
    while (!(_dc = display_lock()));
    graphics_fill_screen(_dc, graphics_make_color(0x00, 0x00, 0x00, 0xFF));
}

void show_screen(void)
{
    display_show(_dc);
}

void put_chr(char chr, unsigned int color, int x, int y)
{
    char temp[4];
    temp[0] = chr;
    temp[1] = 0;
	graphics_set_color(color, 0);
	graphics_draw_text(_dc, x*8, y*8, temp);
}

void put_str(char *str, unsigned int color, int x, int y)
{
	graphics_set_color(color, 0);
	graphics_draw_text(_dc, x*8, y*8, str);
}

void draw_board(void)
{
    unsigned int color = GREEN;
	graphics_draw_line(_dc, STARTX, STARTY, STARTX+LENGTH, STARTY, color);
	graphics_draw_line(_dc, STARTX, STARTY+OFFSET, STARTX+LENGTH-1, STARTY+OFFSET, color);
	graphics_draw_line(_dc, STARTX, STARTY+OFFSET*2, STARTX+LENGTH-1, STARTY+OFFSET*2, color);
	graphics_draw_line(_dc, STARTX, STARTY+OFFSET*3, STARTX+LENGTH, STARTY+OFFSET*3, color);

	graphics_draw_line(_dc, STARTX, STARTY, STARTX, STARTY+LENGTH, color);
	graphics_draw_line(_dc, STARTX+OFFSET, STARTY, STARTX+OFFSET, STARTY+LENGTH-1, color);
	graphics_draw_line(_dc, STARTX+OFFSET*2, STARTY, STARTX+OFFSET*2, STARTY+LENGTH-1, color);
	graphics_draw_line(_dc, STARTX+OFFSET*3, STARTY, STARTX+OFFSET*3, STARTY+LENGTH, color);
}
}

// Game code

using namespace std;

class TicTacToe
{
    public:
        TicTacToe();    // constructor
        int Pick_Player();   // member function
        int Pick_Row();    // member functions
        int Pick_Column();  // member functions
        int Check_Board();  // member function
        void Clear_Board();  // member function
        void Choice_by_Player(int);  // member functions
        void Choice_of_Row(int);   // member functions
        void Choice_of_Column(int);  // member functions
        void Tic_Tac_Toe_Board();  // member functions
        bool Check_Move(int,int); // member functions
    private:
        int row;
        int column;
        int player;
        int board[3][3];
        char display_board[3][3];
};

TicTacToe::TicTacToe()//:row(0):column(0):player(1):board(0)(0):display_board(' ')(' ')
{
    row = 0;
    column = 0;
    player = 1;
    for (int i = 0; i < 3; i++)
    {
        for (int j = 0; j < 3; j++)
        {
            board[i][j] = 0;
            display_board[i][j] = ' ';
        }
    }
}

int TicTacToe::Pick_Player()
{
    return player;
}

int TicTacToe::Pick_Row()
{
    return row;
}

int TicTacToe::Pick_Column()
{
    return column;
}

void TicTacToe::Choice_by_Player(int a)
{
    player = a;
}

void TicTacToe::Choice_of_Row(int b)
{
    row = b;
}

void TicTacToe::Choice_of_Column(int c)
{
    column = c;
}

bool TicTacToe::Check_Move(int row, int column)
{
    if ( board[row][column] != 0 )
    {
        put_str((char*)"Space occupied - Try Again", RED, 20-13, 3);
        return 0;
    }
    else
    {
        board[row][column] = player;
        return 1;
    }
}   // end of Check_Move

int TicTacToe::Check_Board()
{
    int i = 0;

    // check for tie
    for (i = 0; i < 9; i++)
    {
        if (board[i/3][i%3] == 0)
            break;
    } // end of for loop
    if ( i == 9 )
        return 3; // tie game!

    // check rows
    if (( (board[0][0] == player) && (board[0][1] == player) && (board[0][2] == player) ) ||
        ( (board[1][0] == player) && (board[1][1] == player) && (board[1][2] == player) ) ||
        ( (board[2][0] == player) && (board[2][1] == player) && (board[2][2] == player) ))
        return player;

    // check columns
    if (( (board[0][0] == player) && (board[1][0] == player) && (board[2][0] == player) ) ||
        ( (board[0][1] == player) && (board[1][1] == player) && (board[2][1] == player) ) ||
        ( (board[0][2] == player) && (board[1][2] == player) && (board[2][2] == player) ))
        return player;

    // check diagonals
    if (( (board[0][0] == player) && (board[1][1] == player) && (board[2][2] == player) ) ||
        ( (board[0][2] == player) && (board[1][1] == player) && (board[2][0] == player) ))
        return player;

    return 0;
} // end of Check_Board

void TicTacToe::Clear_Board()//:row(0):column(0):player(1):board(0)(0):display_board(' ')(' ')
{
    row = 0;
    column = 0;
    player = 1;
    for (int i = 0; i < 3; i++)
    {
        for (int j = 0; j < 3; j++)
        {
            board[i][j] = 0;
            display_board[i][j] = ' ';
        }
    }
}

void TicTacToe::Tic_Tac_Toe_Board()
{
    draw_board();
    put_str((char*)"Current Player: ", GREEN, 20-8, 21);
    put_chr((player == 1) ? 'X' : 'O', WHITE, 20+8, 21);

    for (int row = 0; row < 3; row ++)
    {
        for (int column = 0; column < 3; column++)
        {
            if ( board[row][column] == 0)
            {
                display_board[row][column] = ' ';
            }
            if ( board[row][column] == 1)
            {
                display_board[row][column] = 'X';
            }
            if ( board[row][column] == 2)
            {
                display_board[row][column] = 'O';
            }
            put_chr(display_board[row][column], WHITE, 20-6+2 + column*4, 6+2 + row*4);
        }  // end of inner for loop
    }  // end of outer for loop

}   // end of Tic_Tac_Toe_Board


int main()
{
    TicTacToe game;
    bool test;
    bool more = true;
    int row = 0;
    int column= 0;
    int player;
    int check = 0;
    short buttons = 0;

    init_n64();

    TicTacToe();

    while ( more )
    {
        clear_screen();
        game.Tic_Tac_Toe_Board();
        player = game.Pick_Player();
        put_str((char*)"Choose a square", GREEN, 20-7, 3);
        show_screen();

        while ( !(A_BUTTON(buttons)) )
        {
            delay(2);
            buttons = get_pad(0);
            if ( START_BUTTON(buttons) )
            {
                // quit
                more = false;
                break;
            }
        }
        while ( A_BUTTON(get_pad(0)) ) delay(2);

        if ( !more )
            break;

        row = 1;
        column = 1;
        if ( DU_BUTTON(buttons) )
            row--;
        else if ( DD_BUTTON(buttons) )
            row++;
        if ( DL_BUTTON(buttons) )
            column--;
        else if ( DR_BUTTON(buttons) )
            column++;
        buttons = 0;

        game.Choice_of_Row(row);
        game.Choice_of_Column(column);

        clear_screen();
        test = game.Check_Move(game.Pick_Row(), game.Pick_Column());
        game.Tic_Tac_Toe_Board(); // update screen after Check_Move sets the player move
        if (test == 0)
        {
            // invalid move
            show_screen();
            delay(120);
            continue;
        }

        check = game.Check_Board();
        if ( (check == 1) || (check == 2) )
        {
            put_chr((check == 1) ? 'X' : 'O', WHITE, 20-4, 3);
            put_str((char*)" wins!", WHITE, 20-3, 3);
            show_screen();
            delay(240);
            // reset the board
            game.Clear_Board();
        }
        else if ( check == 3 )
        {
            put_str((char*)"The game is tied", WHITE, 20-8, 3);
            show_screen();
            delay(240);
            // reset the board
            game.Clear_Board();
        }
        show_screen();
        delay(2);

        if ( player == 1)
        {
            player = 2;
        }
        else
        {
            player = 1;
        }
        game.Choice_by_Player(player);

    } // end of outer while loop

    return 0;
} // end of main function

