#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <stdint.h>
#include <libdragon.h>

/* both ED64 and N64 Myth at same address */
volatile uint32_t *regs_ptr = (uint32_t *) 0xA8040000;

/* ED64 support */

volatile uint8_t spi_cfg;
volatile uint8_t evd_cfg;

uint16_t evd_firm;

#define REG_CFG 0
#define REG_STATUS 1
#define REG_DMA_LEN 2
#define REG_DMA_RAM_ADDR 3
#define REG_MSG 4
#define REG_DMA_CFG 5
#define REG_SPI 6
#define REG_SPI_CFG 7
#define REG_KEY 8
#define REG_SAV_CFG 9
#define REG_SEC 10
#define REG_VER 11

#define REG_CFG_CNT 16
#define REG_CFG_DAT 17
#define REG_MAX_MSG 18
#define REG_CRC 19

#define DCFG_SD_TO_RAM 1
#define DCFG_RAM_TO_SD 2
#define DCFG_FIFO_TO_RAM 3
#define DCFG_RAM_TO_FIFO 4

#define ED_CFG_SDRAM_ON 0
#define ED_CFG_SWAP 1
#define ED_CFG_WR_MOD 2
#define ED_CFG_WR_ADDR_MASK 3

#define SAV_EEP_ON 0
#define SAV_SRM_ON 1
#define SAV_EEP_SIZE 2
#define SAV_SRM_SIZE 3

#define SPI_CFG_SPD0 0
#define SPI_CFG_SPD1 1
#define SPI_CFG_SS 2
#define SPI_CFG_RD 3
#define SPI_CFG_DAT 4
#define SPI_CFG_1BIT 5

uint16_t evd_readReg(uint8_t reg)
{
    volatile uint32_t tmp = regs_ptr[0];

    return regs_ptr[reg];
}

void evd_init()
{
    volatile uint8_t val;

//    wait_ms(1000);
    val = regs_ptr[0];

    spi_cfg = (0 << SPI_CFG_SPD0) | (1 << SPI_CFG_SPD1) | (1 << SPI_CFG_SS);
    evd_cfg = (1 << ED_CFG_SDRAM_ON);

    val = regs_ptr[0];
    regs_ptr[REG_KEY] = 0x1234;

    val = regs_ptr[0];
    regs_ptr[REG_CFG] = evd_cfg;
    val = regs_ptr[0];
    regs_ptr[REG_SPI_CFG] = spi_cfg;

    evd_firm = evd_readReg(REG_VER);
}

/* N64 Myth support */

uint16_t neo_firm;

#define SAVE_IO   0
#define CIC_IO    2
#define ROM_BANK  6
#define ROM_SIZE  4
#define RUN_IO    16
#define INT_IO    18
#define NEO_IO    20
#define SRAM2C_IO 22
#define ROMC_IO   24
#define ROMSW_IO  26
#define SRAM2C_I0 27
#define RST_IO    28
#define CIC_EN    30
#define CPID_IO   32

#define _neo_asic_op(cmd) *(volatile uint32_t *)(0xB2000000 | (cmd<<1))
#define _neo_asic_wop(cmd) *(volatile uint16_t *)(0xB2000000 | (cmd<<1))

void neo_sync_bus(void)
{
    asm volatile
    (
        ".set   push\n"
        ".set   noreorder\n"
        "lui    $t0,0xb000\n"
        "lw     $zero,($t0)\n"
        "jr     $ra\n"
        "nop\n"
        ".set pop\n"
    );
}

// do a Neo Flash ASIC command
static uint16_t _neo_asic_cmd(uint32_t cmd, uint32_t full)
{
    uint32_t data;

    if (full)
    {
        // need to switch rom bus for ASIC operations
        regs_ptr[INT_IO] = 0xFFFFFFFF;            // enable multi-card mode
        neo_sync_bus();
        regs_ptr[ROMSW_IO] = 0x00000000;          // gba mapped to 0xB2000000 - 0xB3FFFFFF
        neo_sync_bus();
        regs_ptr[ROM_BANK] = 0x00000000;
        neo_sync_bus();
        regs_ptr[ROM_SIZE] = 0x000C000C;          // bank size = 256Mbit
        neo_sync_bus();
    }

    // at this point, assumes rom bus in a mode that allows ASIC operations

    regs_ptr[NEO_IO] = 0xFFFFFFFF;                // 16 bit mode
    neo_sync_bus();

    /* do unlocking sequence */
    _neo_asic_op(0x00FFD200);
    _neo_asic_op(0x00001500);
    _neo_asic_op(0x0001D200);
    _neo_asic_op(0x00021500);
    _neo_asic_op(0x00FE1500);
    /* do ASIC command */
    data = _neo_asic_op(cmd);

    regs_ptr[NEO_IO] = 0x00000000;                // 32 bit mode
    neo_sync_bus();

    return (uint16_t)(data & 0x0000FFFF);
}

static uint32_t neo_get_cpld(void)
{
    uint32_t data;

    regs_ptr[INT_IO] = 0xFFFFFFFF;                // enable multi-card mode
    neo_sync_bus();

    regs_ptr[CPID_IO] = 0xFFFFFFFF;               // enable CPLD ID read
    neo_sync_bus();
    data = *(volatile uint32_t *)0xB0000000;
    neo_sync_bus();
    regs_ptr[CPID_IO] = 0x00000000;               // disable CPLD ID read
    neo_sync_bus();

    return data;
}

static void neo_enable_sd(void)
{
    // Neo2-SD (compatible with Neo2-Pro for 16MB of psram)
    _neo_asic_cmd(0x00E21500, 1);                 // GBA CARD WE ON !
    _neo_asic_cmd(0x00372682, 0);                 // set cr = game flash and write enabled, enable SD interface
    _neo_asic_cmd(0x00DAAF4E, 0);                 // select psram
    _neo_asic_cmd(0x00EE0630, 0);                 // set cr1 = enable extended address bus

    regs_ptr[ROMSW_IO] = 0xFFFFFFFF;              // gba mapped to 0xB0000000 - 0xB3FFFFFF
    neo_sync_bus();
    regs_ptr[ROM_SIZE] = 0x000C000C;              // bank size = 256Mbit (128Mbit psram : 128Mbit SD iface)
    neo_sync_bus();
}


int main(void)
{
    /* enable interrupts (on the CPU) */
    init_interrupts();

    /* Initialize peripherals */
    display_init( RESOLUTION_320x240, DEPTH_32_BPP, 2, GAMMA_NONE, ANTIALIAS_RESAMPLE );
    console_init();
    controller_init();

    console_set_render_mode(RENDER_MANUAL);

    /* try to init EV64 */
    evd_init();

    if (evd_firm & 0xFF00)
    {
        /* print firmware rev */
        console_clear();
        printf("\n\n ED64 Firmware Rev: 0x%04X\n", evd_firm);
        console_render();
    }
    else
    {
        neo_enable_sd();
        neo_firm = neo_get_cpld() & 15;

        console_clear();
        printf("\n\n Myth Firmware Rev: 0x%04X\n", neo_firm);
        console_render();
    }

    /* Main loop test */
    while(1) ;

    return 0;
}
