OUTPUT_ARCH(mips)
EXTERN (_start)
ENTRY (_start)
__DYNAMIC  =  0;

/*
 * The memory map looks like this:
 * +--------------------+ <- 0x80000400
 * | .text              |
 * |        _etext      |
 * |        ctor list   | the ctor and dtor lists are for
 * |        dtor list   | C++ support
 * +--------------------+
 * | .data              | initialized data goes here
 * |        _edata      |
 * +--------------------+
 * | .bss               |
 * |        __bss_start | start of bss, cleared by crt0
 * |        _end        | start of heap, used by sbrk()
 * +--------------------+
 * .                    .
 * .                    .
 * .                    .
 * |        __stack     | top of stack (for Master SH2)
 * +--------------------+ <- 0x80400000
 */

MEMORY
{
    ram (wx) : ORIGIN = 0x80000400, LENGTH = 0x003FFC00
}

/*
 * Allocate the stack to be at the top of memory, since the stack
 * grows down
 */

PROVIDE (__stack = 0x80400000);

SECTIONS
{
  .text 0x80000400 :
  AT ( 0x00000000 )
  {
    *(.boot)
    . = ALIGN(16);
    __text_start = .;
    *(.text)
    *(.text.*)
    *(.gnu.linkonce.t.*)

    . = ALIGN(16);
    __INIT_SECTION__ = .;
    KEEP (*(.init))
    LONG (0x03E00008)	/* jr $ra */
    LONG (0x00000000)	/* nop */
    . = ALIGN(16);
    __FINI_SECTION__ = .;
    KEEP (*(.fini))
    LONG (0x03E00008)	/* jr $ra */
    LONG (0x00000000)	/* nop */
  } > ram

  .eh_frame_hdr : { *(.eh_frame_hdr) } > ram
  .eh_frame : { KEEP (*(.eh_frame)) } > ram
  .gcc_except_table : { *(.gcc_except_table) } > ram
  .jcr : { KEEP (*(.jcr)) } > ram

  .ctors    :
  {
    . = ALIGN(16);
     __CTOR_LIST__ = .;
    ___CTOR_LIST__ = .;
    LONG((__CTOR_END__ - __CTOR_LIST__) / 4 - 2)
    KEEP (*(SORT(.ctors.*)))
    KEEP (*(.ctors))
    LONG(0)
    __CTOR_END__ = .;
  } > ram

  .dtors    :
  {
    . = ALIGN(16);
    __DTOR_LIST__ = .;
    ___DTOR_LIST__ = .;
    LONG((__DTOR_END__ - __DTOR_LIST__) / 4 - 2)
    KEEP (*(SORT(.dtors.*)))
    KEEP (*(.dtors))
     LONG(0)
    __DTOR_END__ = .;
  } > ram

  . = .;
  .rodata : {
    *(.rdata)
    *(.rodata)
    *(.rodata.*)
    *(.gnu.linkonce.r.*)
  } > ram
  . = ALIGN(16);
  __text_end = .;

  .data : {
    __data_start = . ;
    *(.data)
    *(.data.*)
    *(.gnu.linkonce.d.*)
    CONSTRUCTORS
  } > ram

  . = ALIGN(8);
  _gp = . + 0x8000;
  __global = _gp;
  .lit8 : {
    *(.lit8)
  } > ram
  .lit4 : {
    *(.lit4)
  } > ram
  .sdata : {
    *(.sdata)
    *(.sdata.*)
    *(.gnu.linkonce.s.*)
  } > ram
  . = ALIGN(8);
  __data_end = .;
  .sbss : {
    __bss_start = .;
    *(.sbss)
    *(.sbss.*)
    *(.gnu.linkonce.sb.*)
    *(.scommon)
  } > ram

  .bss : {
    *(.bss)
    *(.bss.*)
    *(.gnu.linkonce.b.*)
    *(COMMON)
  } > ram
  end = .;
  _end = ALIGN (16);
  __end = _end;
  __bss_end = .;
}
