#
# Defines for N64 RSP
#

# CPU access to RSP DMEM

    .equ RSP_DMEM_ADDR,         0xA4000000

# CPU access to RSP IMEM

    .equ RSP_IMEM_ADDR,         0xA4001000

# CPU access to RSP registers
#   The registers are the base address plus the COP0 index times 4

    .equ RSP_BASE_ADDR,         0xA4040000

# CPU access to RDP registers
#   The registers are the base address plus the (COP0 index - 8) times 4

    .equ RDP_BASE_ADDR,         0xA4100000

# CPU access to the PC in the RSP
#   Halt the RSP, set the DMEM/IMEM, set the RSP PC, then release it.

    .equ RSP_PRG_COUNTER,       0xA4080000

# RSP COP0 registers

    .equ RSP_DMEM_ADDR,         $0
    .equ RSP_DRAM_DMA_ADDR,     $1
    .equ RSP_READ_DMA_LEN,      $2
    .equ RSP_WRITE_DMA_LEN,     $3
    .equ RSP_STATUS,            $4
    .equ RSP_DMA_FULL,          $5
    .equ RSP_DMA_BUSY,          $6
    .equ RSP_SEMAPHORE,         $7

    .equ RDP_CMD_DMA_START,     $8
    .equ RDP_CMD_DMA_END,       $9
    .equ RDP_CMD_DMA_CURR,      $10
    .equ RDP_CMD_STATUS,        $11
    .equ RDP_CLK_COUNTER,       $12
    .equ RDP_BUFF_BUSY_COUNTER, $13
    .equ RDP_PIPE_BUSY_COUNTER, $14
    .equ RDP_TMEM_LOAD_COUNTER, $15

# RSP Status bits (RD)

    .equ RSP_STATUS_HALT,       0x0001
    .equ RSP_STATUS_BROKE,      0x0002
    .equ RSP_STATUS_DMABUSY,    0x0004
    .equ RSP_STATUS_DMAFULL,    0x0008
    .equ RSP_STATUS_IOFULL,     0x0010
    .equ RSP_STATUS_SSTEP,      0x0020
    .equ RSP_STATUS_INTR_BREAK, 0x0040
    .equ RSP_STATUS_SIGNAL0,    0x0080
    .equ RSP_STATUS_SIGNAL1,    0x0100
    .equ RSP_STATUS_SIGNAL2,    0x0200
    .equ RSP_STATUS_SIGNAL3,    0x0400
    .equ RSP_STATUS_SIGNAL4,    0x0800
    .equ RSP_STATUS_SIGNAL5,    0x1000
    .equ RSP_STATUS_SIGNAL6,    0x2000
    .equ RSP_STATUS_SIGNAL7,    0x4000

# RSP Status bits (WR)

    .equ RSP_STATUS_CLR_HALT,    0x00000001
    .equ RSP_STATUS_SET_HALT,    0x00000002
    .equ RSP_STATUS_CLR_BROKE,   0x00000004 # BROKE is set by executing break on the RSP
    .equ RSP_STATUS_CLR_INTER,   0x00000008
    .equ RSP_STATUS_SET_INTER,   0x00000010
    .equ RSP_STATUS_CLR_SSTEP,   0x00000020
    .equ RSP_STATUS_SET_SSTEP,   0x00000040
    .equ RSP_STATUS_CLR_INTRBRK, 0x00000080
    .equ RSP_STATUS_SET_INTRBRK, 0x00000100
    .equ RSP_STATUS_CLR_SIGNAL0, 0x00000200
    .equ RSP_STATUS_SET_SIGNAL0, 0x00000400
    .equ RSP_STATUS_CLR_SIGNAL1, 0x00000800
    .equ RSP_STATUS_SET_SIGNAL1, 0x00001000
    .equ RSP_STATUS_CLR_SIGNAL2, 0x00002000
    .equ RSP_STATUS_SET_SIGNAL2, 0x00004000
    .equ RSP_STATUS_CLR_SIGNAL3, 0x00008000
    .equ RSP_STATUS_SET_SIGNAL3, 0x00010000
    .equ RSP_STATUS_CLR_SIGNAL4, 0x00020000
    .equ RSP_STATUS_SET_SIGNAL4, 0x00040000
    .equ RSP_STATUS_CLR_SIGNAL5, 0x00080000
    .equ RSP_STATUS_SET_SIGNAL5, 0x00100000
    .equ RSP_STATUS_CLR_SIGNAL6, 0x00200000
    .equ RSP_STATUS_SET_SIGNAL6, 0x00400000
    .equ RSP_STATUS_CLR_SIGNAL7, 0x00800000
    .equ RSP_STATUS_SET_SIGNAL7, 0x01000000


#
# Macros for N64 RSP Vector Unit - info taken from MESS N64 RSP emulation
#

    .equ    gr0,    0
    .equ    gr1,    1
    .equ    gr2,    2
    .equ    gr3,    3
    .equ    gr4,    4
    .equ    gr5,    5
    .equ    gr6,    6
    .equ    gr7,    7
    .equ    gr8,    8
    .equ    gr9,    9
    .equ    gr10,   10
    .equ    gr11,   11
    .equ    gr12,   12
    .equ    gr13,   13
    .equ    gr14,   14
    .equ    gr15,   15
    .equ    gr16,   16
    .equ    gr17,   17
    .equ    gr18,   18
    .equ    gr19,   19
    .equ    gr20,   20
    .equ    gr21,   21
    .equ    gr22,   22
    .equ    gr23,   23
    .equ    gr24,   24
    .equ    gr25,   25
    .equ    gr26,   26
    .equ    gr27,   27
    .equ    gr28,   28
    .equ    gr29,   29
    .equ    gr30,   30
    .equ    gr31,   31

    .equ    vr0,    0
    .equ    vr1,    1
    .equ    vr2,    2
    .equ    vr3,    3
    .equ    vr4,    4
    .equ    vr5,    5
    .equ    vr6,    6
    .equ    vr7,    7
    .equ    vr8,    8
    .equ    vr9,    9
    .equ    vr10,   10
    .equ    vr11,   11
    .equ    vr12,   12
    .equ    vr13,   13
    .equ    vr14,   14
    .equ    vr15,   15
    .equ    vr16,   16
    .equ    vr17,   17
    .equ    vr18,   18
    .equ    vr19,   19
    .equ    vr20,   20
    .equ    vr21,   21
    .equ    vr22,   22
    .equ    vr23,   23
    .equ    vr24,   24
    .equ    vr25,   25
    .equ    vr26,   26
    .equ    vr27,   27
    .equ    vr28,   28
    .equ    vr29,   29
    .equ    vr30,   30
    .equ    vr31,   31

    .equ    vco,    0
    .equ    vcc,    1
    .equ    vce,    2

# Vector elements
#   0: 0, 1, 2, 3, 4, 5, 6, 7 == none
#   1: 0, 1, 2, 3, 4, 5, 6, 7 == ???
#   2: 0, 0, 2, 2, 4, 4, 6, 6 == 0q
#   3: 1, 1, 3, 3, 5, 5, 7, 7 == 1q
#   4: 0, 0, 0, 0, 4, 4, 4, 4 == 0h
#   5: 1, 1, 1, 1, 5, 5, 5, 5 == 1h
#   6: 2, 2, 2, 2, 6, 6, 6, 6 == 2h
#   7: 3, 3, 3, 3, 7, 7, 7, 7 == 3h
#   8: 0, 0, 0, 0, 0, 0, 0, 0 == 0
#   9: 1, 1, 1, 1, 1, 1, 1, 1 == 1
#   A: 2, 2, 2, 2, 2, 2, 2, 2 == 2
#   B: 3, 3, 3, 3, 3, 3, 3, 3 == 3
#   C: 4, 4, 4, 4, 4, 4, 4, 4 == 4
#   D: 5, 5, 5, 5, 5, 5, 5, 5 == 5
#   E: 6, 6, 6, 6, 6, 6, 6, 6 == 6
#   F: 7, 7, 7, 7, 7, 7, 7, 7 == 7


# Opcode legend:
#    D = Vector register
#    T = General purpose register
#    I = Vector (byte) Index

# MFC2 rT,vD[I]
#
# 31       25      20      15      10     6         0
# ---------------------------------------------------
# | 010010 | 00000 | TTTTT | DDDDD | IIII | 0000000 |
# ---------------------------------------------------
#
# Move two bytes from byte index of vector register to RSP general register

    .macro  mfvr general, vector, index
    .long   0x12<<26|0<<21|\general<<16|\vector<<11|\index<<7
    .endm

# CFC2 rT,cD
#
# 31       25      20      15      10            0
# ------------------------------------------------
# | 010010 | 00010 | TTTTT | DDDDD | 00000000000 |
# ------------------------------------------------
#
# Move two bytes from vector flags to RSP general register

    .macro  mfvf general, vector
    .long   0x12<<26|2<<21|\general<<16|\vector<<11
    .endm

# MTC2 rT,vD[I]
#
# 31       25      20      15      10     6         0
# ---------------------------------------------------
# | 010010 | 00100 | TTTTT | DDDDD | IIII | 0000000 |
# ---------------------------------------------------
#
# Move two bytes from RSP general register to byte index of vector register

    .macro  mtvr general, vector, index
    .long   0x12<<26|4<<21|\general<<16|\vector<<11|\index<<7
    .endm

# CTC2 rT,cD
#
# 31       25      20      15      10            0
# ------------------------------------------------
# | 010010 | 00110 | TTTTT | DDDDD | 00000000000 |
# ------------------------------------------------
#
# Move two bytes from RSP general register to vector flags

    .macro  mtvf general, vector
    .long   0x12<<26|6<<21|\general<<16|\vector<<11
    .endm


# Opcode legend:
#    B = Base register
#    T = Destination vector
#    I = Destination (byte) Index

# LBV vT[I],Offset(rB)
#
# 31       25      20      15      10     6        0
# --------------------------------------------------
# | 110010 | BBBBB | TTTTT | 00000 | IIII | Offset |
# --------------------------------------------------
#
# Load 1 byte to vector byte index

    .macro  lbv dest, index, offset, base
    .long   0x32<<26|\base<<21|\dest<<16|0<<11|\index<<7|\offset
    .endm

# LSV vT[I],Offset(rB)
#
# 31       25      20      15      10     6        0
# --------------------------------------------------
# | 110010 | BBBBB | TTTTT | 00001 | IIII | Offset |
# --------------------------------------------------
#
# Loads 2 bytes starting from vector byte index

    .macro  lsv dest, index, offset, base
    .long   0x32<<26|\base<<21|\dest<<16|1<<11|\index<<7|\offset>>1
    .endm

# LLV vT[I],Offset(rB)
#
# 31       25      20      15      10     6        0
# --------------------------------------------------
# | 110010 | BBBBB | TTTTT | 00010 | IIII | Offset |
# --------------------------------------------------
#
# Loads 4 bytes starting from vector byte index

    .macro  llv dest, index, offset, base
    .long   0x32<<26|\base<<21|\dest<<16|2<<11|\index<<7|\offset>>2
    .endm

# LDV vT[I],Offset(rB)
#
# 31       25      20      15      10     6        0
# --------------------------------------------------
# | 110010 | BBBBB | TTTTT | 00011 | IIII | Offset |
# --------------------------------------------------
#
# Loads 8 bytes starting from vector byte index

    .macro  ldv dest, index, offset, base
    .long   0x32<<26|\base<<21|\dest<<16|3<<11|\index<<7|\offset>>3
    .endm

# LQV vT[I],Offset(rB)
#
# 31       25      20      15      10     6        0
# --------------------------------------------------
# | 110010 | BBBBB | TTTTT | 00100 | IIII | Offset |
# --------------------------------------------------
#
# Loads up to 16 bytes starting from vector byte index

    .macro  lqv dest, index, offset, base
    .long   0x32<<26|\base<<21|\dest<<16|4<<11|\index<<7|\offset
    .endm

# LRV vT[I],Offset(rB)
#
# 31       25      20      15      10     6        0
# --------------------------------------------------
# | 110010 | BBBBB | TTTTT | 00101 | IIII | Offset |
# --------------------------------------------------
#
# Stores up to 16 bytes starting from right side until 16-byte boundary

    .macro  lrv dest, index, offset, base
    .long   0x32<<26|\base<<21|\dest<<16|5<<11|\index<<7|\offset
    .endm

# LPV vT[I],Offset(rB)
#
# 31       25      20      15      10     6        0
# --------------------------------------------------
# | 110010 | BBBBB | TTTTT | 00110 | IIII | Offset |
# --------------------------------------------------
#
# Loads a byte as the upper 8 bits of each element

    .macro  lpv dest, index, offset, base
    .long   0x32<<26|\base<<21|\dest<<16|6<<11|\index<<7|\offset
    .endm

# LHV vT[I],Offset(rB)
#
# 31       25      20      15      10     6        0
# --------------------------------------------------
# | 110010 | BBBBB | TTTTT | 01000 | IIII | Offset |
# --------------------------------------------------
#
# Loads a byte as the bits 14-7 of each element, with 2-byte stride

    .macro  lhv dest, index, offset, base
    .long   0x32<<26|\base<<21|\dest<<16|8<<11|\index<<7|\offset
    .endm

# LFV vT[I],Offset(rB)
#
# 31       25      20      15      10     6        0
# --------------------------------------------------
# | 110010 | BBBBB | TTTTT | 01001 | IIII | Offset |
# --------------------------------------------------
#
# Loads a byte as the bits 14-7 of upper or lower quad, with 4-byte stride

    .macro  lfv dest, index, offset, base
    .long   0x32<<26|\base<<21|\dest<<16|9<<11|\index<<7|\offset
    .endm

# LWV vT[I],Offset(rB)
#
# 31       25      20      15      10     6        0
# --------------------------------------------------
# | 110010 | BBBBB | TTTTT | 01010 | IIII | Offset |
# --------------------------------------------------
#
# Loads the full 128-bit vector starting from vector byte index and wrapping to index 0
# after byte index 15

    .macro  lwv dest, index, offset, base
    .long   0x32<<26|\base<<21|\dest<<16|10<<11|\index<<7|\offset
    .endm

# LTV vT[I],Offset(rB)
#
# 31       25      20      15      10     6        0
# --------------------------------------------------
# | 110010 | BBBBB | TTTTT | 01011 | IIII | Offset |
# --------------------------------------------------
#
# Loads one element to maximum of 8 vectors, while incrementing element index

    .macro  ltv dest, index, offset, base
    .long   0x32<<26|\base<<21|\dest<<16|11<<11|\index<<7|\offset
    .endm


# Opcode legend:
#    B = Base register
#    T = Source vector
#    I = Source Index

# SBV vT[I],Offset(rB)
#
# 31       25      20      15      10     6        0
# --------------------------------------------------
# | 111010 | BBBBB | TTTTT | 00000 | IIII | Offset |
# --------------------------------------------------
#
# Stores 1 byte from vector byte index

    .macro  sbv dest, index, offset, base
    .long   0x3A<<26|\base<<21|\dest<<16|0<<11|\index<<7|\offset
    .endm

# SSV vT[I],Offset(rB)
#
# 31       25      20      15      10     6        0
# --------------------------------------------------
# | 111010 | BBBBB | TTTTT | 00001 | IIII | Offset |
# --------------------------------------------------
#
# Stores 2 bytes starting from vector byte index

    .macro  ssv dest, index, offset, base
    .long   0x3A<<26|\base<<21|\dest<<16|1<<11|\index<<7|\offset>>1
    .endm

# SLV vT[I],Offset(rB)
#
# 31       25      20      15      10     6        0
# --------------------------------------------------
# | 111010 | BBBBB | TTTTT | 00010 | IIII | Offset |
# --------------------------------------------------
#
# Stores 4 bytes starting from vector byte index

    .macro  slv dest, index, offset, base
    .long   0x3A<<26|\base<<21|\dest<<16|2<<11|\index<<7|\offset>>2
    .endm

# SDV vT[I],Offset(rB)
#
# 31       25      20      15      10     6        0
# --------------------------------------------------
# | 111010 | BBBBB | TTTTT | 00011 | IIII | Offset |
# --------------------------------------------------
#
# Stores 8 bytes starting from vector byte index

    .macro  sdv dest, index, offset, base
    .long   0x3A<<26|\base<<21|\dest<<16|3<<11|\index<<7|\offset>>3
    .endm

# SQV vT[I],Offset(rB)
#
# 31       25      20      15      10     6        0
# --------------------------------------------------
# | 111010 | BBBBB | TTTTT | 00100 | IIII | Offset |
# --------------------------------------------------
#
# Stores up to 16 bytes starting from vector byte index until 16-byte boundary

    .macro  sqv dest, index, offset, base
    .long   0x3A<<26|\base<<21|\dest<<16|4<<11|\index<<7|\offset
    .endm

# SRV vT[I],Offset(rB)
#
# 31       25      20      15      10     6        0
# --------------------------------------------------
# | 111010 | BBBBB | TTTTT | 00101 | IIII | Offset |
# --------------------------------------------------
#
# Stores up to 16 bytes starting from right side until 16-byte boundary

    .macro  srv dest, index, offset, base
    .long   0x3A<<26|\base<<21|\dest<<16|5<<11|\index<<7|\offset
    .endm

# SPV vT[I],Offset(rB)
#
# 31       25      20      15      10     6        0
# --------------------------------------------------
# | 111010 | BBBBB | TTTTT | 00110 | IIII | Offset |
# --------------------------------------------------
#
# Stores upper 8 bits of each element

    .macro  spv dest, index, offset, base
    .long   0x3A<<26|\base<<21|\dest<<16|6<<11|\index<<7|\offset
    .endm

# SUV vT[I],Offset(rB)
#
# 31       25      20      15      10     6        0
# --------------------------------------------------
# | 111010 | BBBBB | TTTTT | 00111 | IIII | Offset |
# --------------------------------------------------
#
# Stores bits 14-7 of each element

    .macro  suv dest, index, offset, base
    .long   0x3A<<26|\base<<21|\dest<<16|7<<11|\index<<7|\offset
    .endm

# SHV vT[I],Offset(rB)
#
# 31       25      20      15      10     6        0
# --------------------------------------------------
# | 111010 | BBBBB | TTTTT | 01000 | IIII | Offset |
# --------------------------------------------------
#
# Stores bits 14-7 of each element, with 2-byte stride

    .macro  shv dest, index, offset, base
    .long   0x3A<<26|\base<<21|\dest<<16|8<<11|\index<<7|\offset
    .endm

# SFV vT[I],Offset(rB)
#
# 31       25      20      15      10     6        0
# --------------------------------------------------
# | 111010 | BBBBB | TTTTT | 01001 | IIII | Offset |
# --------------------------------------------------
#
# Stores bits 14-7 of upper or lower quad, with 4-byte stride

    .macro  sfv dest, index, offset, base
    .long   0x3A<<26|\base<<21|\dest<<16|9<<11|\index<<7|\offset
    .endm

# SWV vT[I],Offset(rB)
#
# 31       25      20      15      10     6        0
# --------------------------------------------------
# | 111010 | BBBBB | TTTTT | 01010 | IIII | Offset |
# --------------------------------------------------
#
# Stores the full 128-bit vector starting from vector byte index and wrapping to index 0
# after byte index 15

    .macro  swv dest, index, offset, base
    .long   0x3A<<26|\base<<21|\dest<<16|10<<11|\index<<7|\offset
    .endm

# STV vT[I],Offset(rB)
#
# 31       25      20      15      10     6        0
# --------------------------------------------------
# | 111010 | BBBBB | TTTTT | 01011 | IIII | Offset |
# --------------------------------------------------
#
# Stores one element from maximum of 8 vectors, while incrementing element index

    .macro  stv dest, index, offset, base
    .long   0x3A<<26|\base<<21|\dest<<16|11<<11|\index<<7|\offset
    .endm


# Opcode legend:
#    E = VS2 element type
#    S = VS2, Source vector 2
#    T = VS1, Source vector 1
#    D = Destination vector

# VMULF vD,vT,vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | TTTTT | DDDDD | 000000 |
# ------------------------------------------------------
#
# Multiplies signed integer by signed integer * 2

    .macro  vmulf dest, src1, src2, elem
    .long   0x12<<26|1<<25|\elem<<21|\src2<<16|\src1<<11|\dest<<6|0
    .endm

# VMULU vD,vT,vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | TTTTT | DDDDD | 000001 |
# ------------------------------------------------------
#

    .macro  vmulu dest, src1, src2, elem
    .long   0x12<<26|1<<25|\elem<<21|\src2<<16|\src1<<11|\dest<<6|1
    .endm

# VMUDL vD,vT,vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | TTTTT | DDDDD | 000100 |
# ------------------------------------------------------
#
# Multiplies unsigned fraction by unsigned fraction
# Stores the higher 16 bits of the 32-bit result to accumulator
# The low slice of accumulator is stored into destination element

    .macro  vmudl dest, src1, src2, elem
    .long   0x12<<26|1<<25|\elem<<21|\src2<<16|\src1<<11|\dest<<6|4
    .endm

# VMUDM vD,vT,vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | TTTTT | DDDDD | 000101 |
# ------------------------------------------------------
#
# Multiplies signed integer by unsigned fraction
# The result is stored into accumulator
# The middle slice of accumulator is stored into destination element

    .macro  vmudm dest, src1, src2, elem
    .long   0x12<<26|1<<25|\elem<<21|\src2<<16|\src1<<11|\dest<<6|5
    .endm

# VMUDN vD,vT,vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | TTTTT | DDDDD | 000110 |
# ------------------------------------------------------
#
# Multiplies unsigned fraction by signed integer
# The result is stored into accumulator
# The low slice of accumulator is stored into destination element

    .macro  vmudn dest, src1, src2, elem
    .long   0x12<<26|1<<25|\elem<<21|\src2<<16|\src1<<11|\dest<<6|6
    .endm

# VMUDH vD,vT,vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | TTTTT | DDDDD | 000111 |
# ------------------------------------------------------
#
# Multiplies signed integer by signed integer
# The result is stored into highest 32 bits of accumulator, the low slice is zero
# The highest 32 bits of accumulator is saturated into destination element

    .macro  vmudh dest, src1, src2, elem
    .long   0x12<<26|1<<25|\elem<<21|\src2<<16|\src1<<11|\dest<<6|7
    .endm

# VMACF vD,vT,vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | TTTTT | DDDDD | 001000 |
# ------------------------------------------------------
#
# Multiplies signed integer by signed integer * 2
# The result is added to accumulator

    .macro  vmacf dest, src1, src2, elem
    .long   0x12<<26|1<<25|\elem<<21|\src2<<16|\src1<<11|\dest<<6|8
    .endm

# VMACU vD,vT,vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | TTTTT | DDDDD | 001001 |
# ------------------------------------------------------
#

    .macro  vmacu dest, src1, src2, elem
    .long   0x12<<26|1<<25|\elem<<21|\src2<<16|\src1<<11|\dest<<6|9
    .endm

# VMADL vD,vT,vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | TTTTT | DDDDD | 001100 |
# ------------------------------------------------------
#
# Multiplies unsigned fraction by unsigned fraction
# Adds the higher 16 bits of the 32-bit result to accumulator
# The low slice of accumulator is stored into destination element

    .macro  vmadl dest, src1, src2, elem
    .long   0x12<<26|1<<25|\elem<<21|\src2<<16|\src1<<11|\dest<<6|12
    .endm

# VMADM vD,vT,vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | TTTTT | DDDDD | 001101 |
# ------------------------------------------------------
#
# Multiplies signed integer by unsigned fraction
# The result is added into accumulator
# The middle slice of accumulator is stored into destination element

    .macro  vmadm dest, src1, src2, elem
    .long   0x12<<26|1<<25|\elem<<21|\src2<<16|\src1<<11|\dest<<6|13
    .endm

# VMADN vD,vT,vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | TTTTT | DDDDD | 001110 |
# ------------------------------------------------------
#
# Multiplies unsigned fraction by signed integer
# The result is added into accumulator
# The low slice of accumulator is stored into destination element

    .macro  vmadn dest, src1, src2, elem
    .long   0x12<<26|1<<25|\elem<<21|\src2<<16|\src1<<11|\dest<<6|14
    .endm

# VMADH vD,vT,vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | TTTTT | DDDDD | 001111 |
# ------------------------------------------------------
#
# Multiplies signed integer by signed integer
# The result is added into highest 32 bits of accumulator, the low slice is zero
# The highest 32 bits of accumulator is saturated into destination element

    .macro  vmadh dest, src1, src2, elem
    .long   0x12<<26|1<<25|\elem<<21|\src2<<16|\src1<<11|\dest<<6|15
    .endm

# VADD vD,vT,vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | TTTTT | DDDDD | 010000 |
# ------------------------------------------------------
#
# Adds two vector registers and carry flag, the result is saturated to -32768/32767

    .macro  vadd dest, src1, src2, elem
    .long   0x12<<26|1<<25|\elem<<21|\src2<<16|\src1<<11|\dest<<6|16
    .endm

# VSUB vD,vT,vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | TTTTT | DDDDD | 010001 |
# ------------------------------------------------------
#
# Subtracts two vector registers and carry flag, the result is saturated to -32768/32767

    .macro  vsub dest, src1, src2, elem
    .long   0x12<<26|1<<25|\elem<<21|\src2<<16|\src1<<11|\dest<<6|17
    .endm

# VABS vD,vT,vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | TTTTT | DDDDD | 010011 |
# ------------------------------------------------------
#
# Changes the sign of source register 2 if source register 1 is negative and stores
# the result to destination register

    .macro  vabs dest, src1, src2, elem
    .long   0x12<<26|1<<25|\elem<<21|\src2<<16|\src1<<11|\dest<<6|19
    .endm

# VADDC vD,vT,vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | TTTTT | DDDDD | 010100 |
# ------------------------------------------------------
#
# Adds two vector registers, the carry out is stored into carry register

    .macro  vaddc dest, src1, src2, elem
    .long   0x12<<26|1<<25|\elem<<21|\src2<<16|\src1<<11|\dest<<6|20
    .endm

# VSUBC vD,vT,vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | TTTTT | DDDDD | 010101 |
# ------------------------------------------------------
#
# Subtracts two vector registers, the carry out is stored into carry register

    .macro  vsubc dest, src1, src2, elem
    .long   0x12<<26|1<<25|\elem<<21|\src2<<16|\src1<<11|\dest<<6|21
    .endm

# VSAW vD,ACCUM_H/M/L
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | TTTTT | DDDDD | 011101 |
# ------------------------------------------------------
#
# Stores high (8), middle (9) or low (10) slice of accumulator to destination vector

    .macro  vsaw dest, elem
    .long   0x12<<26|1<<25|\elem<<21|0<<16|0<<11|\dest<<6|29
    .endm

# VLT vD,vT,vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | TTTTT | DDDDD | 100000 |
# ------------------------------------------------------
#
# Sets compare flags if elements in VS1 are less than VS2
# Moves the element in VS2 to destination vector

    .macro  vlt dest, src1, src2, elem
    .long   0x12<<26|1<<25|\elem<<21|\src2<<16|\src1<<11|\dest<<6|32
    .endm

# VEQ vD,vT,vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | TTTTT | DDDDD | 100001 |
# ------------------------------------------------------
#
# Sets compare flags if elements in VS1 are equal with VS2
# Moves the element in VS2 to destination vector

    .macro  veq dest, src1, src2, elem
    .long   0x12<<26|1<<25|\elem<<21|\src2<<16|\src1<<11|\dest<<6|33
    .endm

# VNE vD,vT,vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | TTTTT | DDDDD | 100010 |
# ------------------------------------------------------
#
# Sets compare flags if elements in VS1 are not equal with VS2
# Moves the element in VS2 to destination vector

    .macro  vne dest, src1, src2, elem
    .long   0x12<<26|1<<25|\elem<<21|\src2<<16|\src1<<11|\dest<<6|34
    .endm

# VGE vD,vT,vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | TTTTT | DDDDD | 100011 |
# ------------------------------------------------------
#
# Sets compare flags if elements in VS1 are greater or equal with VS2
# Moves the element in VS2 to destination vector

    .macro  vge dest, src1, src2, elem
    .long   0x12<<26|1<<25|\elem<<21|\src2<<16|\src1<<11|\dest<<6|35
    .endm

# VCL vD,vT,vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | TTTTT | DDDDD | 100100 |
# ------------------------------------------------------
#
# Vector clip low

    .macro  vcl dest, src1, src2, elem
    .long   0x12<<26|1<<25|\elem<<21|\src2<<16|\src1<<11|\dest<<6|36
    .endm

# VCH vD,vT,vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | TTTTT | DDDDD | 100101 |
# ------------------------------------------------------
#
# Vector clip high

    .macro  vch dest, src1, src2, elem
    .long   0x12<<26|1<<25|\elem<<21|\src2<<16|\src1<<11|\dest<<6|37
    .endm

# VCR vD,vT,vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | TTTTT | DDDDD | 100110 |
# ------------------------------------------------------
#
# Vector clip reverse

    .macro  vcr dest, src1, src2, elem
    .long   0x12<<26|1<<25|\elem<<21|\src2<<16|\src1<<11|\dest<<6|38
    .endm

# VMRG vD,vT,vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | TTTTT | DDDDD | 100111 |
# ------------------------------------------------------
#
# Merges two vectors according to compare flags

    .macro  vmrg dest, src1, src2, elem
    .long   0x12<<26|1<<25|\elem<<21|\src2<<16|\src1<<11|\dest<<6|39
    .endm

# VAND vD,vT,vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | TTTTT | DDDDD | 101000 |
# ------------------------------------------------------
#
# Bitwise AND of two vector registers

    .macro  vand dest, src1, src2, elem
    .long   0x12<<26|1<<25|\elem<<21|\src2<<16|\src1<<11|\dest<<6|40
    .endm

# VNAND vD,vT,vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | TTTTT | DDDDD | 101001 |
# ------------------------------------------------------
#
# Bitwise NOT AND of two vector registers

    .macro  vnand dest, src1, src2, elem
    .long   0x12<<26|1<<25|\elem<<21|\src2<<16|\src1<<11|\dest<<6|41
    .endm

# VOR vD,vT,vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | TTTTT | DDDDD | 101010 |
# ------------------------------------------------------
#
# Bitwise OR of two vector registers

    .macro  vor dest, src1, src2, elem
    .long   0x12<<26|1<<25|\elem<<21|\src2<<16|\src1<<11|\dest<<6|42
    .endm

# VNOR vD,vT,vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | TTTTT | DDDDD | 101011 |
# ------------------------------------------------------
#
# Bitwise NOT OR of two vector registers

    .macro  vnor dest, src1, src2, elem
    .long   0x12<<26|1<<25|\elem<<21|\src2<<16|\src1<<11|\dest<<6|43
    .endm

# VXOR vD,vT,vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | TTTTT | DDDDD | 101100 |
# ------------------------------------------------------
#
# Bitwise XOR of two vector registers

    .macro  vxor dest, src1, src2, elem
    .long   0x12<<26|1<<25|\elem<<21|\src2<<16|\src1<<11|\dest<<6|44
    .endm

# VNXOR vD,vT,vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | TTTTT | DDDDD | 101101 |
# ------------------------------------------------------
#
# Bitwise NOT XOR of two vector registers

    .macro  vnxor dest, src1, src2, elem
    .long   0x12<<26|1<<25|\elem<<21|\src2<<16|\src1<<11|\dest<<6|45
    .endm

# VRCP vD[F],vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | ?FFFF | DDDDD | 110000 |
# ------------------------------------------------------
#
# Calculates reciprocal

    .macro  vrcp dest, delem, src, selem
    .long   0x12<<26|1<<25|\selem<<21|\src<<16|\delem<<11|\dest<<6|48
    .endm

# VRCPL vD[F],vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | ?FFFF | DDDDD | 110001 |
# ------------------------------------------------------
#
# Calculates reciprocal low part

    .macro  vrcpl dest, delem, src, selem
    .long   0x12<<26|1<<25|\selem<<21|\src<<16|\delem<<11|\dest<<6|49
    .endm

# VRCPH vD[F],vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | ?FFFF | DDDDD | 110010 |
# ------------------------------------------------------
#
# Calculates reciprocal high part

    .macro  vrcph dest, delem, src, selem
    .long   0x12<<26|1<<25|\selem<<21|\src<<16|\delem<<11|\dest<<6|50
    .endm

# VMOV vD[F],vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | ?FFFF | DDDDD | 110011 |
# ------------------------------------------------------
#
# Moves element from vector to destination vector

    .macro  vmov dest, delem, src, selem
    .long   0x12<<26|1<<25|\selem<<21|\src<<16|\delem<<11|\dest<<6|51
    .endm

# VRSQ vD[F],vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | ?FFFF | DDDDD | 110100 |
# ------------------------------------------------------
#
# Calculates reciprocal square-root

    .macro  vrsq dest, delem, src, selem
    .long   0x12<<26|1<<25|\selem<<21|\src<<16|\delem<<11|\dest<<6|52
    .endm

# VRSQL vD[F],vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | ?FFFF | DDDDD | 110101 |
# ------------------------------------------------------
#
# Calculates reciprocal square-root low part

    .macro  vrsql dest, delem, src, selem
    .long   0x12<<26|1<<25|\selem<<21|\src<<16|\delem<<11|\dest<<6|53
    .endm

# VRSQH vD[F],vS[E]
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | ?FFFF | DDDDD | 110110 |
# ------------------------------------------------------
#
# Calculates reciprocal square-root high part

    .macro  vrsqh dest, delem, src, selem
    .long   0x12<<26|1<<25|\selem<<21|\src<<16|\delem<<11|\dest<<6|54
    .endm

# VNOP
#
# 31       25  24     20      15      10      5        0
# ------------------------------------------------------
# | 010010 | 1 | EEEE | SSSSS | ?FFFF | DDDDD | 110111 |
# ------------------------------------------------------
#
# Vector null instruction

    .macro  vnop
    .long   0x12<<26|1<<25|55
    .endm

