#
# Default RSP Library code - by  Chilly Willy
#
# Assemble using "-G 0 -EB -mips3" options
#
    .include "rsp.i"

# Mixer structure

    .struct 0
mixer_base:
    .struct mixer_base + 4
mixer_index:
    .struct mixer_index + 4
mixer_pitch:
    .struct mixer_pitch + 4
mixer_lpstart:
    .struct mixer_lpstart + 4
mixer_lpend:
    .struct mixer_lpend + 4
mixer_left:
    .struct mixer_left + 2
mixer_right:
    .struct mixer_right + 2
mixer_sample:
    .struct mixer_sample + 2
mixer_step:
    .struct mixer_step + 2
mixer_pindex:
    .struct mixer_pindex + 4
mixer_sizeof:

# Mixer variables (offset into common area)

    .struct 0
mixer:
    .struct mixer + mixer_sizeof
mix_src:
    .struct mix_src + 8
mix_buf:
    .struct mix_buf + 2*2*2*2           # two stereo 16-bit samples, double buffered








#
# RSP Data Section
#
    .data

# General data here

    .short  0, 0, 0, 0, 0, 0, 0, 0      # for clearing vector registers

# Mixer data here

# From the ADPCM reference source
index_table:
    .byte   -1, -1, -1, -1, 2, 4, 6, 8, -1, -1, -1, -1, 2, 4, 6, 8

# From the ADPCM reference source
step_table:
    .short  7, 8, 9, 10, 11, 12, 13, 14, 16, 17
    .short  19, 21, 23, 25, 28, 31, 34, 37, 41, 45
    .short  50, 55, 60, 66, 73, 80, 88, 97, 107, 118
    .short  130, 143, 157, 173, 190, 209, 230, 253, 279, 307
    .short  337, 371, 408, 449, 494, 544, 598, 658, 724, 796
    .short  876, 963, 1060, 1166, 1282, 1411, 1552, 1707, 1878, 2066
    .short  2272, 2499, 2749, 3024, 3327, 3660, 4026, 4428, 4871, 5358
    .short  5894, 6484, 7132, 7845, 8630, 9493, 10442, 11487, 12635, 13899
    .short  15289, 16818, 18500, 20350, 22385, 24623, 27086, 29794, 32767

# 2D Graphics data here

# 3D Graphics data here



    .balign 8
common_area:


#
# Function arguments, copied from job
#
    .org    0x0FC0
    .comm   fn_args, 0x40


#
# RSP code section
#
    .text
    .set    gp=32
    .set    sym32
    .set    nomips16
    .set    noreorder
    .set    noat

    .equ    ra,         $31
    .equ    sp,         $30

# Mixer register equates

    .equ    mixMixer,   $28             # address of mixer_t in dram (must be 8-byte aligned)
    .equ    mixBuffer,  $27             # address of audio buffer in dram (must be 8-byte aligned)
    .equ    mixCount,   $26             # number of samples to mix (must be even)

    .equ    mixBase,    $25             # pointer to start of source samples in dram
    .equ    mixIndex,   $24             # 20.12 fixed point sample index
    .equ    mixPitch,   $23             # 20.12 fixed point sample increment for index
    .equ    mixLpStart, $22             # 20.12 fixed point sample index of loop start
    .equ    mixLpEnd,   $21             # 20.12 fixed point sample index of loop end

    .equ    mixSample,  $20             # current sample data for ADPCM
    .equ    mixStepIx,  $19             # current sample step_index for ADPCM
    .equ    mixPIndex,  $18             # previous index for ADPCM

    .equ    mixOffset,  $17             # offset into double-buffered mix buffer in dmem
    .equ    mixFrame,   $16             # frame number of current sample in dram

    .equ    mix_source, $15             # address of frame of source samples in dmem
    .equ    mix_buffer, $14             # address of double-buffered mix buffer in dmem

# 2D Graphics register equates




#
# Module entries
#
    .int    mixer_name
    .int    mixer_fns

    .int    g2d_name
    .int    g2d_fns

    .int    g3d_name
    .int    g3d_fns

    .int    0

#
# Support functions
#

dma_not_full:
    mfc0    $1,RSP_STATUS
    andi    $1,$1,RSP_STATUS_DMAFULL
    bne     $1,$0,dma_not_full
    nop
    jr      ra
    nop

dma_not_busy:
    mfc0    $1,RSP_STATUS
    andi    $1,$1,RSP_STATUS_DMABUSY
    bne     $1,$0,dma_not_busy
    nop
    jr      ra
    nop

dma_read:
    mtc0    $2,RSP_DMEM_ADDR
    mtc0    $3,RSP_DRAM_DMA_ADDR
    mtc0    $4,RSP_READ_DMA_LEN
    jr      ra
    nop

dma_write:
    mtc0    $2,RSP_DMEM_ADDR
    mtc0    $3,RSP_DRAM_DMA_ADDR
    mtc0    $4,RSP_WRITE_DMA_LEN
    jr      ra
    nop

read_mixer:
    move    $5,ra
    jal     dma_not_full                # wait for dma slot to open
    nop

    addiu   $2,$29,%lo(mixer)           # address in dmem for mixer_t
    move    $3,mixMixer                 # address in dram of mixer_t
    jal     dma_read                    # read mixer into dmem
    ori     $4,$0,mixer_sizeof-1        # mixer_sizeof bytes

    jal     dma_not_busy                # wait on dma
    nop

    jr      $5
    nop

write_mixer:
    move    $5,ra
    jal     dma_not_full                # wait for dma slot to open
    nop

    addiu   $2,$29,%lo(mixer)           # address in dmem of mixer_t
    move    $3,mixMixer                 # address in dram for mixer_t
    jal     dma_write                   # write mixer back to dram
    ori     $4,$0,mixer_sizeof-1        # mixer_sizeof bytes

    jal     dma_not_busy                # wait on dma
    nop

    jr      $5
    nop

read_frame:
    move    $5,ra
    jal     dma_not_full
    nop

    move    $2,mix_source               # address of source samples in dmem
    sll     $3,mixFrame,3               # frame number to address in dram
    jal     dma_read                    # read source samples into dmem from dram
    ori     $4,$0,8-1                   # 8 bytes

    jal     dma_not_busy
    nop

    jr      $5
    nop

read_mix_buf:
    move    $5,ra
    jal     dma_not_full                # wait for dma slot to open
    nop

    addu    $2,mix_buffer,mixOffset     # address in dmem of double-buffered mix buffer
    move    $3,mixBuffer                # address in dram of audio buffer
    jal     dma_read                    # read current buffer into mix_buf
    ori     $4,$0,8-1                   # 8 bytes

    jr      $5
    nop

# fetch 4 bit sample, do sample fetch DMA if needed

fetch_4b:
    srl     $1,mixIndex,12              # fixed point sample index to int sample index (nibble)
    beq     $1,mixPIndex,8f             # same as last sample index, use already decoded sample
    move    mixPIndex,$1

    # fetch nibble and decode

    srl     $1,mixIndex,12+1            # fixed point to int sample index (+1 because 4 bits per sample)
    addu    $1,$1,mixBase               # address of sample in dram
    srl     $1,$1,3                     # frame number containing sample
    beq     mixFrame,$1,0f              # == mixFrame => still have samples
    nop

    move    $6,ra                       # save return address
    jal     read_frame                  # read a frame from dram (16 samples)
    move    mixFrame,$1                 # new frame
    move    ra,$6                       # restore return address
0:
    srl     $1,mixIndex,12+1            # fixed point to int sample index (+1 because 4 bits per sample)
    addu    $1,$1,mixBase               # address of sample in dram
    andi    $1,$1,7                     # byte offset of sample in frame
    addu    $1,$1,mix_source            # address of sample in dmem
    lbu     $2,0($1)
    srl     $1,mixIndex,12              # fixed point sample index to int sample index (nibble)
    andi    $1,$1,1                     # which nibble
    xori    $1,$1,1
    sll     $1,$1,2                     # 0 -> 4, 1 -> 0
    srlv    $2,$2,$1
    andi    $2,$2,15                    # nibble

    # decode nibble into sample

    sll     $3,mixStepIx,1
    lh      $3,%lo(step_table)($3)      # step = step_table[c->step_index]

    lb      $1,%lo(index_table)($2)
    addu    mixStepIx,mixStepIx,$1      # step_index = c->step_index + index_table[(unsigned)nibble];
    bgez    mixStepIx,0f
    nop
    move    mixStepIx,$0                # if (step_index < 0) step_index = 0;
0:
    addiu   $1,mixStepIx,-88
    blez    $1,1f
    nop
    ori     mixStepIx,$0,88             # else if (step_index > 88) step_index = 88;
1:
    move    $4,$3                       # diff = step
    sll     $3,$3,1
    andi    $1,$2,1
    beq     $1,$0,2f
    nop
    addu    $4,$4,$3                    # if (nibble & 1) diff += (step << 1)
2:
    sll     $3,$3,1
    andi    $1,$2,2
    beq     $1,$0,3f
    nop
    addu    $4,$4,$3                    # if (nibble & 2) diff += (step << 2)
3:
    sll     $3,$3,1
    andi    $1,$2,4
    beq     $1,$0,4f
    nop
    addu    $4,$4,$3                    # if (nibble & 4) diff += (step << 3)
4:
    srl     $4,$4,3                     # diff >>= 3;
    andi    $1,$2,8
    beq     $1,$0,5f
    nop
    j       6f
    subu    mixSample,mixSample,$4      # if (nibble & 8) sample -= diff
5:
    addu    mixSample,mixSample,$4      # else sample += diff
6:
    addiu   $1,mixSample,-32767
    blez    $1,7f
    nop
    addiu   mixSample,$0,32767          # clamp to 32767
7:
    ori     $1,$0,0x8000
    addu    $1,$1,mixSample
    bgez    $1,8f
    addiu   mixSample,$0,-32768         # clamp to -32768
8:
    jr      ra
    move    $2,mixSample

# fetch 8 bit sample, do sample fetch DMA if needed

fetch_8b:
    srl     $1,mixIndex,12              # fixed point to int sample index
    addu    $1,$1,mixBase               # address of sample in dram
    srl     $1,$1,3                     # frame number containing sample
    beq     mixFrame,$1,0f              # == mixFrame => still have samples
    nop

    move    $6,ra                       # save return address
    jal     read_frame                  # read a frame from dram (8 samples)
    move    mixFrame,$1                 # new frame
    move    ra,$6                       # restore return address
0:
    srl     $1,mixIndex,12              # fixed point to int sample index
    addu    $1,$1,mixBase               # address of sample in dram
    andi    $1,$1,7                     # offset of sample in frame
    addu    $1,$1,mix_source            # address of sample in dmem
    lb      $2,0($1)
    jr      ra
    sll     $2,$2,8                     # convert 8-bit sample to 16-bit sample

# fetch 16 bit sample, do sample fetch DMA if needed

fetch_16b:
    srl     $1,mixIndex,12-1            # fixed point to int sample index (-1 because 2 bytes per sample)
    addu    $1,$1,mixBase               # address of sample in dram
    srl     $1,$1,3                     # frame number containing sample
    beq     mixFrame,$1,0f              # == mixFrame => still have samples
    nop

    move    $6,ra                       # save return address
    jal     read_frame                  # read a frame from dram (4 samples)
    move    mixFrame,$1                 # new frame
    move    ra,$6                       # restore return address
0:
    srl     $1,mixIndex,12-1            # fixed point to int sample index (-1 because 2 bytes per sample)
    addu    $1,$1,mixBase               # address of sample in dram
    andi    $1,$1,6                     # offset of sample in frame
    addu    $1,$1,mix_source            # address of sample in dmem
    jr      ra
    lh      $2,0($1)

mixer_prolog:
    move    $6,ra                       # save return address
    ori     sp,$0,%lo(fn_args)
    ori     $29,$0,%lo(common_area)
    lw      mixMixer,0(sp)              # mixer (must be 8-byte aligned)
    lw      mixBuffer,4(sp)             # buffer (must be 8-byte aligned)
    lw      mixCount,8(sp)              # count (must be even)

    jal     read_mixer                  # read mixer into dmem
    nop

    addiu   $2,$29,%lo(mixer)
    lw      mixBase,mixer_base($2)
    lw      mixIndex,mixer_index($2)
    lw      mixPitch,mixer_pitch($2)
    lw      mixLpStart,mixer_lpstart($2)
    lw      mixLpEnd,mixer_lpend($2)
    llv     vr2,0,mixer_left,gr2
    llv     vr2,4,mixer_left,gr2        # vr2 = left:right:left:right:-:-:-:-

    move    mixOffset,$0
    addiu   mixFrame,$0,-1              # force load of frame on first fetch

    addiu   mix_source,$29,%lo(mix_src) # address of source samples in dmem
    addiu   mix_buffer,$29,%lo(mix_buf) # address of double-buffered mix buffer in dmem

    jr      $6
    nop

mixer_epilog:
    addiu   $2,$29,%lo(mixer)           # address of mixer_t in dmem
    sw      mixBase,mixer_base($2)      # update base field of mixer
    sw      mixIndex,mixer_index($2)    # update index field of mixer

    jal     write_mixer                 # write mixer back to dram
    nop

    break

mix_samples:
    # load from buffer
    addu    $2,mix_buffer,mixOffset     # address of double-buffered mix buffer in dmem
    ldv     vr1,0,0,gr2                 # load two 16-bit stereo samples into vr1 from mix buffer
    # scale samples
    vmulf   vr0,vr0,vr2,0               # accum = ((vr0 * vr2) << 17) + 32768, vr0 = 16-bit clamped(accum31-16)
    # mix with buffer
    mtvf    gr0,vco                     # clear carry out flags
    vadd    vr0,vr0,vr1,0               # vr0 = 16-bit clamped(vr0 + vr1 + vco)
    # save to buffer
    sdv     vr0,0,0,gr2                 # store two 16-bit stereo mixed samples to mix buffer

    # write mix buffer
    move    $5,ra
    jal     dma_not_full                # wait for dma slot to open
    nop

#   addu    $2,mix_buffer,mixOffset     # address of double-buffered mix buffer in dmem
    xori    mixOffset,mixOffset,8       # flip mix buffer
    move    $3,mixBuffer                # address of audio buffer in dram
    jal     dma_write                   # write mix buffer to audio buffer in dram
    ori     $4,$0,8-1                   # 8 bytes

    jr      $5
    nop

#
# Mixer Module
#

mixer_name:
    .asciz  "mixer"

    .balign 4
mixer_fns:
    .int    mix_4b
    .int    mix_8b
    .int    mix_16b
    .int    effects

#
# Mix a 4-bit (ADPCM) mixer channel to a buffer
#

mix_4b:
    jal     mixer_prolog
    nop
    # load extra ADPCM registers
    lh      mixSample,mixer_sample($2)
    lh      mixStepIx,mixer_step($2)
    lw      mixPIndex,mixer_pindex($2)

loop_4b:
    jal     read_mix_buf                # get samples currently in buffer
    nop

    # mix two signed 4-bit samples with mix buffer
    # first sample
    jal     fetch_4b                    # fetch sample in $2
    nop
    mtvr    gr2,vr0,0

    # increment index and check length
    addu    mixIndex,mixIndex,mixPitch
    subu    $1,mixIndex,mixLpEnd
    bltz    $1,0f
    nop
    # end of sample
    mtvr    gr0,vr0,2                   # force second sample = 0
    j       1f
    move    mixBase,$0                  # disable mixer
0:
    # second sample
    jal     fetch_4b                    # fetch sample in $2
    nop
    mtvr    gr2,vr0,2

    # increment index and check length
    addu    mixIndex,mixIndex,mixPitch
    subu    $1,mixIndex,mixLpEnd
    bltz    $1,1f
    nop
    # end of sample
    move    mixBase,$0                  # disable mixer
1:
    jal     mix_samples
    nop

    beq     mixBase,$0,2f               # mixer disabled
    addiu   mixCount,mixCount,-2
    bgtz    mixCount,loop_4b
    addiu   mixBuffer,mixBuffer,8
2:
    addiu   $2,$29,%lo(mixer)
    sw      mixBase,mixer_base($2)      # update base field of mixer
    sw      mixIndex,mixer_index($2)    # update index field of mixer
    # save extra ADPCM registers
    sh      mixSample,mixer_sample($2)
    sh      mixStepIx,mixer_step($2)
    sw      mixPIndex,mixer_pindex($2)

    jal     write_mixer                 # write mixer back to dram
    nop

    break

#
# Mix an 8-bit (PCM) mixer channel to a buffer
#

mix_8b:
    jal     mixer_prolog
    nop

loop_8b:
    jal     read_mix_buf                # get samples currently in buffer
    nop

    # mix two signed 8-bit samples with mix buffer
    # first sample
    jal     fetch_8b                    # fetch sample in $2
    nop
    mtvr    gr2,vr0,0

    # increment index and check length
    addu    mixIndex,mixIndex,mixPitch
    subu    $1,mixIndex,mixLpEnd
    bltz    $1,0f
    nop
    # end of sample, loop?
    bne     mixLpStart,$0,0f
    addu    mixIndex,$1,mixLpStart
    # no looping, disable mixer
    mtvr    gr0,vr0,2                   # force second sample = 0
    j       1f
    move    mixBase,$0
0:
    # second sample
    jal     fetch_8b                    # fetch sample in $2
    nop
    mtvr    gr2,vr0,2

    # increment index and check length
    addu    mixIndex,mixIndex,mixPitch
    subu    $1,mixIndex,mixLpEnd
    bltz    $1,1f
    nop
    # end of sample, loop?
    bne     mixLpStart,$0,1f
    addu    mixIndex,$1,mixLpStart
    # no looping, disable mixer
    move    mixBase,$0
1:
    jal     mix_samples
    nop

    beq     mixBase,$0,2f               # mixer disabled
    addiu   mixCount,mixCount,-2
    bgtz    mixCount,loop_8b
    addiu   mixBuffer,mixBuffer,8
2:
    j       mixer_epilog
    nop

#
# Mix a 16-bit (PCM) mixer channel to a buffer
#

mix_16b:
    jal     mixer_prolog
    nop

loop_16b:
    jal     read_mix_buf                # get samples currently in buffer
    nop

    # mix two signed 16-bit samples with mix buffer
    # first sample
    jal     fetch_16b                   # fetch sample in $2
    nop
    mtvr    gr2,vr0,0

    # increment index and check length
    addu    mixIndex,mixIndex,mixPitch
    subu    $1,mixIndex,mixLpEnd
    bltz    $1,0f
    nop
    # end of sample, loop?
    bne     mixLpStart,$0,0f
    addu    mixIndex,$1,mixLpStart
    # no looping, disable mixer
    mtvr    gr0,vr0,2                   # force second sample = 0
    j       1f
    move    mixBase,$0
0:
    # second sample
    jal     fetch_16b                   # fetch sample in $2
    nop
    mtvr    gr2,vr0,2

    # increment index and check length
    addu    mixIndex,mixIndex,mixPitch
    subu    $1,mixIndex,mixLpEnd
    bltz    $1,1f
    nop
    # end of sample, loop?
    bne     mixLpStart,$0,1f
    addu    mixIndex,$1,mixLpStart
    # no looping, disable mixer
    move    mixBase,$0
1:
    jal     mix_samples
    nop

    beq     mixBase,$0,2f               # mixer disabled
    addiu   mixCount,mixCount,-2
    bgtz    mixCount,loop_16b
    addiu   mixBuffer,mixBuffer,8
2:
    j       mixer_epilog
    nop

effects:
    ori     sp,$0,%lo(fn_args)
    ori     $29,$0,%lo(common_area)

    break


#
# 2D Graphics Module
#

g2d_name:
    .asciz  "gfx_2d"

    .balign 4
g2d_fns:
    .int    draw_sprite
    .int    draw_map

draw_sprite:
    ori     sp,$0,%lo(fn_args)
    ori     $29,$0,%lo(common_area)

    break

draw_map:
    ori     sp,$0,%lo(fn_args)
    ori     $29,$0,%lo(common_area)

    break


#
# 3D Graphics Module
#

g3d_name:
    .asciz  "gfx_3d"

    .balign 4
g3d_fns:
    .int    mat_mul
    .int    poly_setup

mat_mul:
    ori     sp,$0,%lo(fn_args)
    ori     $29,$0,%lo(common_area)

    break

poly_setup:
    ori     sp,$0,%lo(fn_args)
    ori     $29,$0,%lo(common_area)

    break

