OUTPUT_ARCH(mips)

/*
 * The memory map looks like this:
 * +--------------------+ <- 0x0000
 * | .data              | initialized data goes here
 * |        _edata      |
 * +--------------------+
 * | .bss               |
 * |        __bss_start | start of bss, cleared by crt0
 * |        __bss_end   | end of bss
 * .                    .
 * |        __stack     | top of stack
 * +--------------------+ <- 0x1000
 * | .text              |
 * |        _etext      |
 * +--------------------+ <- 0x2000
 */

MEMORY
{
    dmem (w) : ORIGIN = 0x0000, LENGTH = 0x1000
    imem (x) : ORIGIN = 0x1000, LENGTH = 0x1000
}

/*
 * Allocate the stack to be at the top of memory, since the stack
 * grows down
 */

PROVIDE (__stack = 0x1000);

SECTIONS
{
  .data 0x0000 : {
    __data_start = . ;
    *(.data)
    . = ALIGN(16);
    __data_end = .;
  } > dmem

  .bss (NOLOAD) : {
    __bss_start = . ;
    *(.bss)
    . = ALIGN(16);
    __bss_end = .;
  } > dmem

  .text 0x1000 : {
    __text_start = .;
    *(.text)
    . = ALIGN(16);
    __text_end = .;
    . = ALIGN(0x1000);
  } > imem
}
